﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO;
using System.Drawing;

namespace Jewels__hra_uživatele_
{
    public class GameWindow : Form
    {
        private Bitmap Bitmap;
        public float FallSpeed = 2;
        public float StoneSize = 64;
        public float Body = 0;

        public Stone[,] Pole;

        public short[] NextStone;
        public List<byte>[] ListNextStone;

        public Point[] MousePoint;

        private Timer Timer;
        public float Time
        {
            get
            {
                return Program.Time;
            }
            set
            {
                Program.Time = value;
            }
        }

        public GameWindow()
        {
            //nacteni ze souboru
            Load();
            if (Pole == null)
            {
                return;
            }
            //velikost
            Bitmap = new Bitmap((int)(StoneSize * 8), (int)(StoneSize * 8));
            ClientSize = Bitmap.Size;
            //nastaveni okna
            FormBorderStyle = FormBorderStyle.FixedSingle;
            MaximizeBox = false;
            Text = "Jewels";
            //nastaveni timer
            Timer = new Timer();
            Timer.Interval = 50;
            Timer.Tick += new EventHandler(Tick);
            Timer.Enabled = true;
        }
        protected override void OnPaint(PaintEventArgs e)
        { e.Graphics.DrawImage(Bitmap, 0, 0); }
        protected override void OnPaintBackground(PaintEventArgs e)
        { }

        public void Load()
        {
            string file = Program.File;
            string[] radek = File.ReadAllLines(file);

            try
            {
                //nacteni pole
                Pole = new Stone[8, 8];
                for (int x = 0; x < 8; x++)
                {
                    for (int y = 0; y < 8; y++)
                    {
                        char cislice = radek[y][x];
                        byte byteCislice = (byte)int.Parse(cislice.ToString());
                        Pole[x, y] = new Stone(byteCislice, new Point(x, y));
                        if (byteCislice < 1 || byteCislice > 6) 
                        {
                            MessageBox.Show("Číslice musí být 1 - 6!");
                            Pole = null;
                            return;
                        }
                    }
                }

                //nacteni dalsich kamenu
                ListNextStone = new List<byte>[8];
                NextStone = new short[8];
                for (int x = 0; x < 8; x++)
                {
                    ListNextStone[x] = new List<byte>();
                    foreach (char cislice in radek[x + 8])
                    {
                        byte byteCislice = (byte)int.Parse(cislice.ToString());
                        ListNextStone[x].Add(byteCislice);
                        if (byteCislice > 6 || byteCislice < 1)
                        {
                            MessageBox.Show("Číslice musí být 1 - 6!");
                            Pole = null;
                            return;
                        }
                    }
                }
            }
            catch
            {
                MessageBox.Show("Nelze načíst!");
                Pole = null;
                return;
            }
        }

        protected override void OnMouseDown(MouseEventArgs e)
        {
            base.OnMouseDown(e);
            MousePoint = new Point[]
            {
                new Point((int)(((float)e.Location.X) / StoneSize), (int)(((float)e.Location.Y) / StoneSize)), Point.Empty
            };
        }
        protected override void OnMouseUp(MouseEventArgs e)
        {
            base.OnMouseUp(e);
            MousePoint = null;
        }
        protected override void OnMouseMove(MouseEventArgs e)
        {
            base.OnMouseMove(e);
            int x = (int)(((float)e.Location.X) / StoneSize);
            int y = (int)(((float)e.Location.Y) / StoneSize);
            if (MousePoint != null)
            {
                if (MousePoint[0].X != x && MousePoint[0].Y == y)
                {
                    if (MousePoint[0].X - 1 == x || MousePoint[0].X + 1 == x)
                    {
                        Stone[,] LastPole = (Stone[,])Pole.Clone();

                        Stone s1 = Pole[MousePoint[0].X, MousePoint[0].Y];
                        Stone s2 = Pole[x, y];
                        Pole[x, y] = s1;
                        Pole[MousePoint[0].X, MousePoint[0].Y] = s2;
                        
                    }
                }
                if (MousePoint[0].X == x && MousePoint[0].Y != y)
                {
                    if (MousePoint[0].Y - 1 == y || MousePoint[0].Y + 1 == y)
                    {
                        Stone[,] LastPole = (Stone[,])Pole.Clone();

                        Stone s1 = Pole[MousePoint[0].X, MousePoint[0].Y];
                        Stone s2 = Pole[x, y];
                        Pole[x, y] = s1;
                        Pole[MousePoint[0].X, MousePoint[0].Y] = s2;
                       
                    }
                }
            }
        }

        private void Tick(object obj, EventArgs e)
        {
            //update
            float fInterval = ((float)Timer.Interval) / 1000;
            UpdateStone(fInterval);
            Time -= fInterval;
            //draw
            Graphics g = Graphics.FromImage(Bitmap);
            g.Clear(Color.LightGray);
            Draw(g);
            g.Dispose();
            Invalidate();
        }
        public void MoveStone()
        {
            for (int x = 0; x < 8; x++)
            {
                for (int y = 0; y < 8; y++)
                {
                    if (Pole[x, y] == null)
                    {
                        //prida kamen
                        if (y - 1 < 0)
                        {
                            Pole[x, y] = new Stone(ListNextStone[x][NextStone[x]], new Point(x, y));
                            Pole[x, y].PointF.Y = -1;
                            NextStone[x] += 1;
                            if (NextStone[x] >= ListNextStone[x].Count)
                            {
                                NextStone[x] = 0;
                            } 
                        }
                        //presune kameny
                        else
                        {
                            Pole[x, y] = Pole[x, y - 1];
                            Pole[x, y].Point.Y = y;
                            Pole[x, y - 1] = null;
                            MoveStone();
                        }
                    }
                }
            }
        }
        public void UpdateSled()
        {
            List<Sled> sledy = GetSled();
            foreach (Sled sled in sledy)
            {
                Body += 3;
                foreach (Stone s in sled.Stone)
                {
                    if (s != null)
                    {
                        Pole[s.Point.X, s.Point.Y] = null;
                    }
                }
            }
        }
        public List<Sled> GetSled()
        {
            List<Sled> sledReturn = new List<Sled>();

            for (int x = 0; x < 8; x++)
            {
                for (int y = 0; y < 8; y++)
                {
                    Sled sRight = new Sled(Pole[x, y]);
                    if(x + 2 < 8)
                    {
                        if (sRight.Color == Pole[x + 1, y].Color && sRight.Color == Pole[x + 2, y].Color) 
                        {
                            sRight.Stone.Add(Pole[x + 2, y]);
                            sRight.Stone.Add(Pole[x + 1, y]);
                            sledReturn.Add(sRight);
                        }
                    }

                    Sled sLeft = new Sled(Pole[x, y]);
                    if (x - 2 > -1)
                    {
                        if (sLeft.Color == Pole[x - 1, y].Color && sLeft.Color == Pole[x - 2, y].Color)
                        {
                            sLeft.Stone.Add(Pole[x - 2, y]);
                            sLeft.Stone.Add(Pole[x - 1, y]);
                            sledReturn.Add(sLeft);
                        }
                    }

                    Sled sTop = new Sled(Pole[x, y]);
                    if (y - 2 > -1)
                    {
                        if (sTop.Color == Pole[x, y - 1].Color && sTop.Color == Pole[x, y - 2].Color)
                        {
                            sTop.Stone.Add(Pole[x, y - 1]);
                            sTop.Stone.Add(Pole[x, y - 2]);
                            sledReturn.Add(sTop);
                        }
                    }

                    Sled sDown = new Sled(Pole[x, y]);
                    if (y + 2 < 8)
                    {
                        if (sDown.Color == Pole[x, y + 1].Color && sDown.Color == Pole[x, y + 2].Color)
                        {
                            sDown.Stone.Add(Pole[x, y + 1]);
                            sDown.Stone.Add(Pole[x, y + 2]);
                            sledReturn.Add(sDown);
                        }
                    }

                }
            }

            return sledReturn;
        }
        public void UpdateStone(float time)
        {
            UpdateSled();
            MoveStone();
            for (int x = 0; x < 8; x++)
            {
                for (int y = 0; y < 8; y++)
                {
                    if (Pole[x, y].PointF.Y < y)
                    {
                        Pole[x, y].PointF.Y += FallSpeed * time;
                        if (Pole[x, y].PointF.Y > y)
                        {
                            Pole[x, y].PointF.Y = y;
                        }
                      
                    }
                }
            }
        }
        public void Draw(Graphics g)
        {
            Pen pen = new Pen(Color.DarkGray);
            for (int x = 0; x < 8; x++)
            {
                for (int y = 0; y < 8; y++)
                {
                    //vykresli kamen
                    Brush brush = new SolidBrush(Color.FromArgb(255, Pole[x, y].Color));
                    g.FillEllipse(brush, Pole[x, y].PointF.X * StoneSize, Pole[x, y].PointF.Y * StoneSize, StoneSize, StoneSize);
                   // g.FillEllipse(brush, Pole[x, y].Point.X * StoneSize, Pole[x, y].Point.Y * StoneSize, StoneSize, StoneSize);
                    brush.Dispose();


                    //vykresli ctverec
                    g.DrawRectangle(pen, x * StoneSize, y * StoneSize, StoneSize, StoneSize);
                }
            }
            pen.Dispose();
        }
    }
}
